unit FasArrayCalObjcts;

interface
uses Classes, SysUtils, FasLocat;

type
  LocationRec = record
    Location: integer;
    Count: integer;
  end;

  CalculationRec = record
    TotalTension, MinTension: Real;
    TotalShear, MinShear: Real;
    Location: integer;
    Count: integer;
  end;

  LocationRecArray = array of LocationRec;
  CalculationRecArray = array of CalculationRec;

  TFasLocRowAndCol = class(TObject)
  private
    FLocationOfFastener: TLocation;
    FDoX: Boolean; // only valid as newnode
    FDirChanged: Boolean;
    FX, FY: Integer;
    FHoldsFastener: Boolean;
    FNextX, FNextY: TFasLocRowAndCol;
    procedure Insert(var AListTop: TFasLocRowAndCol; ANewNode: TFasLocRowAndCol; ALocationTolerance: Integer);
    procedure ChangeDirection;
  public
    function CountOfX: Integer;
    function CountOfY: Integer;
    constructor Create(var AListTop: TFasLocRowAndCol; ADoXFirst: Boolean;
      ANewLocation: TLocation; ALocationTolerance: Integer);
    destructor Destroy; override;
    function ArrayOfXLoc(APos: integer = 0): LocationRecArray;
    function ArrayOfYLoc(APos: integer = 0): LocationRecArray;
    function ArrayOfXCaluclations(APos: integer = 0): CalculationRecArray;
    //function ArrayOfYCaluclations(APos:integer=0): CalculationRecArray;
    procedure CalsInY(var ACalRec: CalculationRec);
    //procedure CalsInX(Var ACalRec:CalculationRec);
    function RightMostRow: TFasLocRowAndCol; //Leftmost is Self ie Top of list
    function TopMostRow: TFasLocRowAndCol; //BottomMost is Self ie Top of list
    procedure PopulateList(AStrings: TStrings; AInX: Boolean);
  end;


implementation
uses Fastener;

function CompareIntegerRange(TestPositiveIfGreater,TestNegativeIfGreater,Range:integer):Integer;
// Returns -1,0,1 if test values differ by more than range

begin
  Result:=0;
  If TestPositiveIfGreater>(TestNegativeIfGreater+Range)then Result:=1;
  If TestNegativeIfGreater>(TestPositiveIfGreater+Range)then Result:=-1;
end;


{ TFasLocRowAndCol }

function TFasLocRowAndCol.ArrayOfXCaluclations(
  APos: integer): CalculationRecArray;
var
  rCalc: CalculationRec;
begin
  if FNextX = nil then
    SetLength(Result, APos + 1)
  else
    Result := FNextX.ArrayOfXCaluclations(APos + 1);
  rCalc := Result[APos];

  rCalc.Location := FX;
  rCalc.Count := CountOfY;
end;

function TFasLocRowAndCol.ArrayOfXLoc(APos: Integer): LocationRecArray;
var
  rLoc: ^LocationRec;
begin
  if FNextX = nil then
    SetLength(Result, APos + 1)
  else
    Result := FNextX.ArrayOfXLoc(APos + 1);
  rLoc := @Result[APos];

  rLoc^.Location := FX;
  rLoc^.Count := CountOfY;
end;

function TFasLocRowAndCol.ArrayOfYLoc(APos: integer): LocationRecArray;
var
  rLoc: ^LocationRec;
begin
  if FNextY = nil then
    SetLength(Result, APos + 1)
  else
    Result := FNextY.ArrayOfYLoc(APos + 1);
  rLoc := @Result[APos];

  rLoc^.Location := FY;
  rLoc^.Count := CountOfX;
end;

procedure TFasLocRowAndCol.CalsInY(var ACalRec: CalculationRec);
var
  Fas: TFastener;
begin
  if FHoldsFastener then
  begin
    if FNextY <> nil then
      FNextY.CalsInY(ACalRec);
    Fas := TFastener(FLocationOfFastener.ImpactObject);
    Fas.TallyCalculationRecord(ACalRec);
  end
  else
    ACalRec.Count := 10 - MaxInt; // if negative then result not complete
end;

procedure TFasLocRowAndCol.ChangeDirection;
begin
  if FDirChanged then
    raise Exception.Create('FDirChanged');
  FDirChanged := True;
  FDoX := not FDoX;
end;

function TFasLocRowAndCol.CountOfX: Integer;
begin
  if FNextX <> nil then
    Result := FNextX.CountOfX + 1
  else
    Result := 1;
end;

function TFasLocRowAndCol.CountOfY: Integer;
begin
  if FNextY <> nil then
    Result := FNextY.CountOfY + 1
  else
    Result := 1;
end;

constructor TFasLocRowAndCol.Create(var AListTop: TFasLocRowAndCol;
  ADoXFirst: Boolean; ANewLocation: TLocation; ALocationTolerance: Integer);
begin
  if ANewLocation = nil then // abort create
    raise Exception.Create('Need Location');

  FLocationOfFastener := ANewLocation;
  fx := Round(ANewLocation.Xloc);
  fy := Round(ANewLocation.Yloc);
  FDoX := ADoXFirst;
  try
    FHoldsFastener := ANewLocation.ImpactObject is TFastener;
  except end; //could be -1

  if AListTop = nil then
    AListTop := Self
  else
    AListTop.Insert(AListTop, Self, ALocationTolerance);
end;

destructor TFasLocRowAndCol.Destroy;
begin
  FNextX.Free;
  FNextY.Free;
  inherited;
end;

procedure TFasLocRowAndCol.Insert(var AListTop: TFasLocRowAndCol;
  ANewNode: TFasLocRowAndCol; ALocationTolerance: Integer);
  {sub} function CompareInThisDirection: Integer;
  begin
    Result := 0;
    case ANewNode.FDoX of
      True: Result := CompareIntegerRange(ANewNode.FX, AListTop.FX, ALocationTolerance);
      False: Result := CompareIntegerRange(ANewNode.FY, AListTop.FY, ALocationTolerance);
    end; //case
  end;

var
  ThisAction: Integer;
begin
  if AListTop <> self then raise Exception.Create('AListTop<>self');

  ThisAction := CompareInThisDirection;
  case ThisAction of
    -1: begin //slot before
        case ANewNode.FDoX of
          True: begin
              ANewNode.FNextX := Self;
              if FNextY <> nil then
                if ANewNode.FDirChanged then
                begin
                  ANewNode.FNextY := FNextY;
                  FNextY := nil;
                end;
            end;
          False: begin
              ANewNode.FNextY := Self;
              if FNextX <> nil then
                if ANewNode.FDirChanged then
                begin
                  ANewNode.FNextX := FNextX;
                  FNextX := nil;
                end;
            end;
        end; //case
        AListTop := ANewNode;
      end;
    0: begin //Match
        ANewNode.ChangeDirection;
        Insert(AListTop, ANewNode, ALocationTolerance);
      end;
    1: //Keep Going
      case ANewNode.FDoX of
        True: if FNextX = nil then
            FNextX := ANewNode
          else
            FNextX.Insert(FNextX, ANewNode, ALocationTolerance);
        False: if FNextY = nil then
            FNextY := ANewNode
          else
            FNextY.Insert(FNextY, ANewNode, ALocationTolerance);
      end; //case
  end; //case
end;


procedure TFasLocRowAndCol.PopulateList(AStrings: TStrings; AInX: Boolean);
begin
  AStrings.AddObject(FLocationOfFastener.TextValue, FLocationOfFastener);
  if AInX then
  begin
    if FNextX <> nil then FNextX.PopulateList(AStrings, Ainx);
  end
  else if FNextY <> nil then FNextY.PopulateList(AStrings, Ainx);
end;

function TFasLocRowAndCol.RightMostRow: TFasLocRowAndCol;
begin
  if FNextX <> nil then
    Result := FNextX.RightMostRow
  else
    Result := self;
end;

function TFasLocRowAndCol.TopMostRow: TFasLocRowAndCol;
begin
  if FNextY <> nil then
    Result := FNextY.TopMostRow
  else
    Result := self;
end;

end.

