unit FormLL;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, FasArrayCalObjcts;

type
  TForm1 = class(TForm)
    ImageToDraw: TImage;
    LbxAllLocs: TListBox;
    BtnNewList: TButton;
    LbxTop: TListBox;
    LbxLeft: TListBox;
    LbxRight: TListBox;
    Lbxbottom: TListBox;
    LblRowInX: TLabel;
    LblRowInY: TLabel;
    procedure BtnNewListClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    XBound, YBound: Real;
    FasLocinX,FasLocInY:TFasLocRowAndCol;
    Procedure FillLbxs;
    procedure FillOneLbx(AStrings:TStrings;AFasLocObj:TFasLocRowAndCol; AInX:Boolean);
    procedure ResetBounds(Ax,Ay:real);
    procedure DrawPlan;
    procedure FreeEveryThing;
    Procedure BuildLocationPlan;
    procedure NewLocations;
    procedure NewRowOfLocations(DoX: Boolean);
    Procedure SetLabels;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses Faslocat, Fastener, Types;

{$R *.dfm}


procedure DrawScale(Canvas: TCanvas; Centre: TPoint; Bounds: TRect; MarksAt: Integer);
const
  m = 3;
var
  Offset, Mark, i: Integer;
  Continue: Boolean;
begin
  with Canvas do
  begin
    PolyLine([Point(Bounds.left, Centre.Y), Point(Bounds.right, Centre.Y)]);
    PolyLine([Point(Centre.X, Bounds.top), Point(Centre.X, Bounds.Bottom)]);
    Continue := true;
    i := 1;
    while Continue do
    begin
      Continue := False;
      offset := i * MarksAt;
      Mark := Centre.Y + offset;
      if Mark < bounds.bottom then
      begin
        PolyLine([Point(Centre.X - m, Mark), Point(Centre.X + m, Mark)]);
        Continue := true;
      end;
      Mark := Centre.Y - offset;
      if Mark > bounds.top then
      begin
        PolyLine([Point(Centre.X - m, Mark), Point(Centre.X + m, Mark)]);
        Continue := true;
      end;
      Mark := Centre.X + offset;
      if Mark < bounds.right then
      begin
        PolyLine([Point(Mark, Centre.Y - m), Point(Mark, Centre.Y + m)]);
        Continue := true;
      end;
      Mark := Centre.X - offset;
      if Mark > bounds.left then
      begin
        PolyLine([Point(Mark, Centre.Y - m), Point(Mark, Centre.Y + m)]);
        Continue := true;
      end;
      I := I + 1;
    end;

  end;
end;


{ TForm1 }

procedure TForm1.BtnNewListClick(Sender: TObject);
begin
  FreeEveryThing;
  NewLocations;
  DrawPlan;
  FillLbxs;
  SetLabels;
end;

procedure TForm1.BuildLocationPlan;
Var
  i:integer;
begin
  FreeAndNil(FasLocinX);
  FreeAndNil(FasLocInY);
  for i:=0 to LbxAllLocs.Count-1 do
    if LbxAllLocs.Items.Objects[i]<>nil then
   begin
     TFasLocRowAndCol.Create(FasLocinX,True,LbxAllLocs.Items.Objects[i] as TLocation,10);
     TFasLocRowAndCol.Create(FasLocinY,False,LbxAllLocs.Items.Objects[i] as TLocation,10);
   end;
end;

procedure TForm1.DrawPlan;
const
  MinScale = 0.01;
  MaxScale = 4;
var
  R, Brect: TRect;
  i, Hi, Scalemm: integer;
  XMax, YMax: real;
  XCenterLine, YCenterLine {,XMax,YMax}: integer;
  XScale, YScale: Real; {pixels per mm}
  SaveBrushColor, SavePenColor: TColor;
  SavePenWidth: Integer;
  SavePenStyle: TPenStyle;
  SaveBrushStyle: TBrushStyle;
  CurrentLocation: TLocation;
  FastenerToDraw: TFastener;
  ScaleCal: LongInt;
  Canvas: TCanvas;
  Inpingments: TStrings;

begin
  Canvas := ImageToDraw.Canvas;
  SaveBrushColor := Canvas.Brush.Color;
  SaveBrushStyle := Canvas.Brush.Style;
  SavePenColor := Canvas.Pen.Color;
  SavePenWidth := Canvas.Pen.Width;
  SavePenStyle := Canvas.Pen.Style;
  try
    Inpingments := LbxAllLocs.Items;
    R := Canvas.ClipRect;
    Canvas.Brush.color := clWhite;
    Canvas.FillRect(r);
    XCenterLine := (r.left + r.right) div 2;
    YCenterLine := (r.top + r.bottom) div 2;
    Xmax := Abs(XBound); {cannot be zero}
    YMax := Abs(YBound);
    XScale := (r.right - r.left - 5) / 2 / Xmax;
    YScale := (r.bottom - r.top - 5) / 2 / Ymax;
    if Xscale < Yscale then
      YScale := Xscale;
    if YScale < MinScale then
      YScale := MinScale;
    if YScale > MaxScale then
      YScale := MaxScale;
    ScaleCal := Round(YCenterLine / YScale / 4);
    if scalecal < 5 then
      Scalemm := 5
    else
      if scalecal < 20 then
        Scalemm := 10
      else
        if scalecal < 80 then
          Scalemm := 50
        else
          if scalecal < 200 then
            Scalemm := 100
          else
            if scalecal < 800 then
              Scalemm := 500
            else
              if scalecal < 2000 then
                Scalemm := 1000
              else
                if scalecal < 8000 then
                  Scalemm := 5000
                else
                  Scalemm := 10000;
    Brect := r;
    Canvas.Brush.color := clLtGray;
    Canvas.FillRect(BRect);
  {Draw Coordinates}
    Canvas.Pen.color := clBlack;
    Canvas.Pen.Width := 1;
    Canvas.Brush.Style := bsClear;
    DrawScale(Canvas, Point(XCenterLine, YCenterLine), r, Round(Scalemm * YScale));
 {Draw All Fasteners}
    Canvas.Brush.Style := BsSolid;
    if (Inpingments <> nil) and (Inpingments.count > 0) then
      for i := 0 to Inpingments.count - 1 do
      begin
        CurrentLocation := Inpingments.objects[i] as TLocation;
        if (CurrentLocation <> nil) then
          with CurrentLocation do
            if ImpactObject <> nil then
            begin
              Canvas.Pen.color := clBlack;
              Canvas.Pen.width := 1;
              FastenerToDraw:=ImpactObject as TFastener;
              FastenerToDraw.DrawPlanNut(Canvas, Round(XLoc * YScale) + XCenterLine,
                Round({Y Location is inverted}-YLoc * YScale) + YCenterLine, 8 * YScale,
                true, true, clYellow);
            end;
      end;
  except {preserve Color allocations}
  end;
  Canvas.Brush.Color := SaveBrushColor;
  Canvas.Brush.Style := SaveBrushStyle;
  Canvas.Pen.Color := SavePenColor;
  Canvas.Pen.Width := SavePenWidth;
  Canvas.Pen.Style := SavePenStyle;
end;


procedure TForm1.FillLbxs;
begin
  BuildLocationPlan;
  FillOneLbx(LbxLeft.Items,FasLocinX,false);
  FillOneLbx(LbxRight.Items,FasLocinX.RightMostRow,false);
  FillOneLbx(Lbxbottom.Items,FasLocinY,True);
  FillOneLbx(LbxTop.Items,FasLocinY.TopMostRow,True);
end;

procedure TForm1.FillOneLbx(AStrings: TStrings;
  AFasLocObj: TFasLocRowAndCol; AInX:Boolean);
begin
  AStrings.Clear;
  AFasLocObj.PopulateList(AStrings,AInX);
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FreeEveryThing;
end;

procedure TForm1.FreeEveryThing;
var
  i: integer;
begin
  XBound := 0;
  YBound := 0;
  FreeAndNil(FasLocinX);
  FreeAndNil(FasLocInY);
  for i := 0 to LbxAllLocs.Count - 1 do
    LbxAllLocs.Items.Objects[i].free;
  LbxAllLocs.Clear;
end;

procedure TForm1.NewLocations;
var
  i: integer;
begin
  for i := 0 to Random(5) + 2 do
    NewRowOfLocations(i mod 2 = 0);
end;

procedure TForm1.NewRowOfLocations(DoX: Boolean);
var
  i, Limit: integer;
  X, Y: Real;
  NewLoc: TLocation;
begin
  X := 0.0000001;
  y := 0.0000001;
  if DoX then
    Y := YBound + 30
  else
    X := XBound + 30;

  Limit := Random(3) + 2;
  if Limit mod 2 = 0 then
  begin
    X := -X;
    Y := -Y;
  end;
  for i := 0 to Limit do
  begin
    if DoX then
      X := (Abs(x) + (Random(5) + 2) * 10) * -(x / Abs(X))
    else
      Y := (Abs(y) + (Random(5) + 2) * 10) * -(y / Abs(Y));
    ResetBounds(X,Y);
    NewLoc := TLocation.Create;
    NewLoc.Xloc := x;
    NewLoc.Yloc := y;
    NewLoc.ImpactObject:=TFastener.Create;
    LbxAllLocs.Items.AddObject(NewLoc.TextValue, NewLoc);
  end;

end;

procedure TForm1.ResetBounds(Ax, Ay: real);
begin
  if Abs(Ax)>XBound then XBound:=Abs(Ax);
  if Abs(Ay)>YBound then YBound:=Abs(Ay);
end;

procedure TForm1.SetLabels;
begin
  if FasLocinX<>nil then
  LblRowInX.Caption:= IntToStr(FasLocinX.CountOfX)+' Rows in X' ;
  LblRowInY.Caption:= IntToStr(FasLocinY.CountOfY)+' Rows in Y' ;
end;

end.

